/**
 * <copyright>
 *
 * Copyright (c) 2006 Chaur G. Wu. All rights reserved. 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * </copyright>
 */

package net.sf.model4lang.boolscript.engine;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.LinkedHashSet;
import javax.script.ScriptContext;
import javax.script.Bindings;
import net.sf.model4lang.boolscript.*;
import net.sf.model4lang.boolscript.parser.BoolScriptParser;
import net.sf.model4lang.boolscript.parser.ParseException;

public class BoolTermEvaluator {

	static final public BoolTerm tTrue = BoolScriptFactory.eINSTANCE.createBoolTerm();
	static final public BoolTerm tFalse = BoolScriptFactory.eINSTANCE.createBoolTerm();
	
	public static Set<Var> getVariables(BoolTerm term)
	{
		Set<Var> varSet = new LinkedHashSet<Var>();
		
		if (term instanceof AndTerm)
		{
			AndTerm andTerm = (AndTerm) term;
			Set<Var> leftVarSet = getVariables(andTerm.getAndLeftTerm());
			Set<Var> rightVarSet = getVariables(andTerm.getAndRightTerm());
			varSet.addAll(leftVarSet);
			varSet.addAll(rightVarSet);
		}
		else if (term instanceof OrTerm)
		{
			OrTerm orTerm = (OrTerm) term;
			Set<Var> leftVarSet = getVariables(orTerm.getOrLeftTerm());
			Set<Var> rightVarSet = getVariables(orTerm.getOrRightTerm());
			varSet.addAll(leftVarSet);
			varSet.addAll(rightVarSet);
		}
		else if (term instanceof Var) {
			varSet.add((Var) term);
		}
		
		return varSet;
	}
	
	public static List<Boolean> evaluate(String script, ScriptContext context) 
		throws ParseException {
		byte[] bytes = script.getBytes();
		ByteArrayInputStream inputStream = new ByteArrayInputStream(bytes);
		return evaluate(inputStream, context);
	}
	
	public static List<Boolean> evaluate(InputStream inputStream, ScriptContext context) 
		throws ParseException {
		BoolScriptParser parser = new BoolScriptParser(inputStream, null);
		List<Boolean> resultList = new ArrayList<Boolean>();
		while (true) {
			BoolTerm term = parser.one_line();
			if (term == null) break;
			
			BoolTerm result = evaluate(term, context);
			if (result == BoolTermEvaluator.tTrue)
				resultList.add(Boolean.TRUE);
			else if(result == BoolTermEvaluator.tFalse)
				resultList.add(Boolean.FALSE);
		}
		
		return resultList;
	}

	public static BoolTerm evaluate(BoolTerm term, ScriptContext context)
	{
		if (term instanceof AndTerm)
		{
			AndTerm andTerm = (AndTerm) term;
			BoolTerm leftResult = evaluate(andTerm.getAndLeftTerm(), context);
			if (leftResult == tFalse) 
				return tFalse;
			else
				return evaluate(andTerm.getAndRightTerm(), context);
		}
		else if (term instanceof OrTerm)
		{
			OrTerm orTerm = (OrTerm) term;
			BoolTerm leftResult = evaluate(orTerm.getOrLeftTerm(), context);
			if (leftResult == tTrue) 
				return tTrue;
			else
				return evaluate(orTerm.getOrRightTerm(), context);
		}
		else if (term instanceof Var) {
			Var var = (Var) term;
			Bindings bindings = context.getBindings(ScriptContext.ENGINE_SCOPE);
			if (!bindings.containsKey(var.getName()))
				throw new IllegalArgumentException("Variable " + var.getName() + " not set.");
			
			Boolean varValue = (Boolean) bindings.get(var.getName());
			if (varValue == Boolean.TRUE)
				return BoolTermEvaluator.tTrue;
			else
				return BoolTermEvaluator.tFalse;
		}
		else
			return term;
	}
}
